/*****************************************************************/
/*                                                               */
/* Nonlinear Fit Example Program                                 */
/*                                                               */
/*****************************************************************/
#define HELP_MSG \
"This program is designed to serve as an example in how to use \n\
the Nonlinear Fit function in the Advanced Analysis Library.  \n\
The user specifies a particular data function to generate data \n\
from and a function to fit the data to.  When the user clicks \n\
on the start button, the program generates an array of points \n\
of the specified size using the selected data function.  \n\
Gaussian noise with a standard deviation set by the user is \n\
added to the generated data to simulate actual acquired data.  \n\
A nonlinear fit is then generated and plotted along with the \n\
data.  The fitting function is selected by the user and the \n\
parameters of the best fit are returned in the boxes on the \n\
panel, along with the mean squared error of the fit.  At the \n\
present time, only two functions are included in this program \n\
for data generation and fitting.  An exponentially damped \n\
sinusoid and an exponential function are available."

#include <cvirte.h>     /* needed if linking executable in external compiler; harmless otherwise */
#include <analysis.h>
#include <ansi_c.h>
#include <userint.h>
#include "nonlnfit.h"

double dampsin (double x, double a[], int ncoef);
double exponential (double x, double a[], int ncoef);
void FitCurveProcedure(void);

static int mainpnl;
static int numpoints = 128;
static double noiselevel = .05;
static int functfit = 1;
static int functtemplate = 1;
static double idealpar2 = 0.2;
static double idealpar1 = 0.03;
static double coefs[2] = {0.03, 0.2};
static double datawave[2048];
static double fitdata[2048];
static double noisewave[2048];
static double xwave[2048];
static int i;
static int seed = 1;
static double meansqerror;
static int autoFitCurve=1;

/*****************************************************************/
/*                                                               */
/* Variable Descriptions                                         */
/*                                                               */
/* 1.  mainpnl - panel handle for the main panel                 */
/* 2.  numpoints - the number of points used in each fitting     */
/*                 default = 128                                 */
/* 3.  noiselevel - standard deviation of the Gaussian noise     */
/*                  added to the generated signal                */
/*                  default = .05                                */
/* 4.  functfit - variable indicating which function to fit the  */
/*                data to                                        */
/*                0 = exponential                                */
/*                1 = damped sinusoid (default)                  */
/* 5.  functtemplate - variable indicating which function to     */
/*                     create data from                          */
/*                     0 = exponential                           */
/*                     1 = damped sinusoid (default)             */
/* 6.  idealpar2 - value of the w parameter of the damped        */
/*                 sinusoid function from which data is created  */
/*                 default = .2                                  */
/* 7.  idealpar1 - value of the a parameter in the functions     */
/*                 from which data is created                    */
/*                 default = .03                                 */
/* 8.  coefs - array containing the parameters to be passed to   */
/*             the functions for fitting purposes                */
/*             default = {.03, .2}                               */
/* 9.  datawave - array containing data points generated by one  */
/*                of the functions                               */
/* 10.  fitdata - array containing the points of the best fit    */
/*                to the data generated by the NonlinearFit      */
/*                function                                       */
/* 11.  noisewave - array containing the Gaussian noise that is  */
/*                  added to the data points prior to fitting    */
/* 12.  xwave - array containing the x axis to send to the       */
/*              fitting function - points start at 0 and end     */
/*              at (numpoints -1), with increments of 1          */
/* 13. i - loop variable                                         */
/* 14. seed - seed used in Gaussian noise generation             */
/* 15. meansqerror - mean squared error of the fit returned by   */
/*                   the fitting function                        */
/*                                                               */
/*****************************************************************/

/*****************************************************************/
/*                                                               */
/* Main Function:                                                */
/*                                                               */
/* The main function loads and displays the main panel.          */
/*                                                               */
/*****************************************************************/
int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    if ((mainpnl = LoadPanel (0, "nonlnfit.uir", MAINPNL)) < 0)
        return -1;
        
    if (autoFitCurve) FitCurveProcedure();

    DisplayPanel (mainpnl);
    RunUserInterface ();
    
    return 0;
}                                   

/*****************************************************************/
/*                                                               */
/*  Getnumpoints Function                                        */
/*                                                               */
/*  The Getnumpoints function is called whenever the user        */
/*  changes the value of the control containing the number of    */
/*  points.  The function gets the new value of the control and  */
/*  puts it in the numpoints variable.                           */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getnumpoints (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (mainpnl, MAINPNL_NUMPOINTS, &numpoints);
            if (autoFitCurve) FitCurveProcedure();

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Number of Points",
                          "Sets the number of data points.");

            break;
    }
    return 0;
}

/*****************************************************************/
/*                                                               */
/*  Getfunctfit Function                                         */
/*                                                               */
/*  The Getfunctfit function is called whenever the user changes */
/*  the value of the control selecting the function to fit the   */
/*  data to.  The function gets the new value of the control and */
/*  puts it in the functfit variable.  The function then changes */
/*  the message in the text box to display the selected function */
/*  in the box.                                                  */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getfunctfit (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (mainpnl, MAINPNL_FUNCTFIT, &functfit);
            if (functfit == 0) {
               ReplaceTextBoxLine (mainpnl, MAINPNL_FITTEMPTEXT, 0,
                                   "Y(X) = Exp(-a*X)");
               
            }
            else {
               ReplaceTextBoxLine (mainpnl, MAINPNL_FITTEMPTEXT, 0,
                                   "Y(X) = Exp(-a*X) * Cos(w*X)");
               
            }   
            if (autoFitCurve) FitCurveProcedure();
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Fitting Function",
                          "Sets the function to fit the data to.");

            break;
    }
    return 0;
}

/****************************************************************/
/*                                                              */
/*  Getfuncttemplate Function                                   */
/*                                                              */
/*  The Getfuncttemplate function is called whenever the user   */
/*  changes the value of the control indicating which function  */
/*  to use when creating data.  The function gets the new value */
/*  of the control and puts it in the functtemplate variable.   */
/*  The function then changes the message in the text box to    */
/*  indicate which function has been selected.                  */
/*                                                              */
/****************************************************************/

int CVICALLBACK Getfuncttemplate (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (mainpnl, MAINPNL_FUNCTTEMPLATE, &functtemplate);
            if (functtemplate == 0) {
               ReplaceTextBoxLine (mainpnl, MAINPNL_DATATEMPTEXT, 0,
                                   "Y(X) = Exp(-a*X)");
               SetInputMode (mainpnl, MAINPNL_IDEALPAR2, 0);
               
            }
            else {
              ReplaceTextBoxLine (mainpnl, MAINPNL_DATATEMPTEXT, 0,
                                  "Y(X) = Exp(-a*X) * Cos(w*X)");
              SetInputMode (mainpnl, MAINPNL_IDEALPAR2, 1);
               
            }   
            if (autoFitCurve) FitCurveProcedure();
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Data Template",
                          "Sets the desired function from which the sample data is drawn.  Noise is added to these points to simulate actual data which differs from a perfect function form.");

            break;
    }
    return 0;
}

/****************************************************************/
/*                                                              */
/*  Getnoiselevel Function                                      */
/*                                                              */
/*  The Getnoiselevel function is called whenever the user      */
/*  changes the value of the control containing the amount of   */
/*  noise added to the data before fitting.  The function gets  */
/*  the new value of the control and puts it in the noiselevel  */
/*  variable.                                                   */
/*                                                              */
/****************************************************************/

int CVICALLBACK Getnoiselevel (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (mainpnl, MAINPNL_NOISELEVEL, &noiselevel);
            if (autoFitCurve) FitCurveProcedure();
            
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Noise Level",
                          "Sets the standard deviation for the noise added to the data prior to fitting.");

            break;
    }
    return 0;
}

                                                                  
/****************************************************************/
/*                                                              */
/*  Getidealpar2 Function                                       */
/*                                                              */
/*  The Getidealpar2 function is called whenever the user       */
/*  changes the value of the control containing the value of w  */
/*  in the damped sinusoid function.  The function gets the new */
/*  value of the control and puts it in the idealpar2 variable. */
/*  This value is also stored in the coefs array for passing to */
/*  the NonlinearFit function.                                  */
/*                                                              */
/****************************************************************/

int CVICALLBACK Getidealpar2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (mainpnl, MAINPNL_IDEALPAR2, &idealpar2);
            coefs[1] = idealpar2;
            if (autoFitCurve) FitCurveProcedure();
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Frequency Value",
                          "Sets the value of w in the Damped Sinusoid function from which the data is drawn.");

            break;
    }
    return 0;
}

/****************************************************************/
/*                                                              */
/*  Getidealpar1 Function                                       */
/*                                                              */
/*  The Getidealpar1 Function is called whenever the user       */
/*  changes the value of the control containing the a parameter */
/*  on the function from which the data is created.  The        */
/*  function gets the new value of the control and puts it in   */
/*  the idealpar1 variable.  The function also puts the value   */
/*  in the coefs array for passing to the NonlinearFit function.*/
/*                                                              */
/****************************************************************/


int CVICALLBACK Getidealpar1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (mainpnl, MAINPNL_IDEALPAR1, &idealpar1);
            coefs[0] = idealpar1;
            if (autoFitCurve) FitCurveProcedure();
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Exponential Parameter",
                          "Sets the value of a in the function from which the data is drawn.");

            break;
    }
    return 0;
}

/****************************************************************/
/*                                                              */
/*  Start Function                                              */
/*                                                              */
/*  The Start function is called whenever the user clicks on    */
/*  the Start button.  The function gets the values from the    */
/*  ideal parameter boxes and updates the coefs array. The      */
/*  array of noise points is created using the noiselevel       */
/*  variable as the standard deviation.  The seed value used in */
/*  creating this array is incremented and resets to 1 if the   */
/*  value increases over 50.  This noise is then added to the   */
/*  data points to simulate actual data.  The array of x points */
/*  from 0 to numpoints-1 is set up.  The function then fits    */
/*  the data to the appropriate function and plots the data     */
/*  with noise and the fit generated with the NonLinear fit     */
/*  function. The function parameter values are returned and    */
/*  displayed in the appropriate controls, as is the mean       */
/*  squared error of the fit.                                   */
/*                                                              */
/****************************************************************/

int CVICALLBACK FitCurve (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            FitCurveProcedure();
            
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Start",
                          "Generates the data and fit according to user settings.");

            break;
    }
    return 0;
}

void FitCurveProcedure(void)
{
    GetCtrlVal (mainpnl, MAINPNL_IDEALPAR1, &idealpar1);
    GetCtrlVal (mainpnl, MAINPNL_IDEALPAR2, &idealpar2);
    coefs[0] = idealpar1;
    coefs[1] = idealpar2;
    
    DeleteGraphPlot (mainpnl, MAINPNL_GRAPH, -1, VAL_IMMEDIATE_DRAW);
    
    seed++;
    if (seed > 50) {
       seed = 1;
    }
    GaussNoise (numpoints, noiselevel, seed, noisewave);
    
    for (i=0; i<numpoints; i++) {
        xwave[i] = i;
        if (functtemplate == 0) datawave[i] = exponential(i,coefs,1);
        if (functtemplate == 1) datawave[i] = dampsin(i,coefs,2);
        datawave[i] = datawave[i] + noisewave[i];
    }
    
    PlotY (mainpnl, MAINPNL_GRAPH, datawave, numpoints, VAL_DOUBLE,
          VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
   
    if (functfit == 0) {
        NonLinearFit (xwave, datawave, fitdata, numpoints, exponential, coefs,
                 1, &meansqerror);
        SetInputMode (mainpnl, MAINPNL_FITPAR2, 0);   
    }
    else {
        NonLinearFit (xwave, datawave, fitdata, numpoints, dampsin, coefs,
                 2, &meansqerror);
        SetInputMode (mainpnl, MAINPNL_FITPAR2, 1);   
        SetCtrlVal (mainpnl, MAINPNL_FITPAR2, coefs[1]);
    }
    
    PlotY (mainpnl, MAINPNL_GRAPH, fitdata, numpoints, VAL_DOUBLE,
          VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_DK_BLUE);
    SetCtrlVal (mainpnl, MAINPNL_FITPAR1, coefs[0]);
    
    SetCtrlVal (mainpnl, MAINPNL_ERROR, meansqerror);
}



/****************************************************************/
/*                                                              */
/*  Quit Function                                               */
/*                                                              */
/*  The Quit function is called whenever the  user clicks on    */
/*  the Quit button.  The execution of the program is           */
/*  terminated.                                                 */
/*                                                              */
/****************************************************************/

int CVICALLBACK Quit (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Quit Program", "Quits the program.");

            break;
    }
    return 0;
}

/****************************************************************/
/*                                                              */
/*  Dampsin Function                                            */
/*                                                              */
/*  The Dampsin function is used to create data points prior to */
/*  fitting.  The function is also used in the NonlinearFit     */
/*  function to generate the fit to the data.  The dampsin      */
/*  function is set up in the manner prescribed by the          */
/*  Nonlinear fit function.  It accepts as arguments a data     */
/*  point x, a parameter array a, and an integer specifying the */
/*  number of parameters used in the function.                  */
/*                                                              */
/****************************************************************/

double dampsin (double x, double a[], int ncoef)
{
   double value;
   
   value = exp(-1*a[0]*x) * cos(a[1] * x);
   return(value);
}   

/****************************************************************/
/*                                                              */
/*  Exponential Function                                        */
/*                                                              */
/*  The exponential function is used to create data points      */
/*  prior to fitting.  The function is also used in the         */
/*  NonlinearFit function to generate the fit to the data.  The */
/*  syntax of the function is similar to the dampsin function   */
/*  which is described above.                                   */
/*                                                              */
/****************************************************************/

double exponential (double x, double a[], int ncoef)
{
   double value;
   
   value = exp(-1*a[0]*x);
   return (value);
}   

/*****************************************************************/
/*                                                               */
/* Help Button Callback                                          */
/*                                                               */
/*****************************************************************/
int CVICALLBACK HelpCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_RIGHT_CLICK:
        case EVENT_COMMIT:
            MessagePopup ("Nonlinear Fit Example Program",HELP_MSG);
        break;
    }
    return 0;
}


int CVICALLBACK SetAutoFitState (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal(panel, control, &autoFitCurve);

            break;
    }
    return 0;
}
